@@ -495,6 +495,12 @@ class LogRecord(object):
         self.extra = ExtraDict(self.extra)
         return self
 
+    def _format_message(self, msg, *args, **kwargs):
+        """Called if the record's message needs to be formatted.
+        Subclasses can implement their own formatting.
+        """
+        return msg.format(*args, **kwargs)
+
     @cached_property
     def message(self):
         """The formatted message."""
@@ -502,11 +508,11 @@ class LogRecord(object):
             return self.msg
         try:
             try:
-                return self.msg.format(*self.args, **self.kwargs)
+                return self._format_message(self.msg, *self.args, **self.kwargs)
             except UnicodeDecodeError:
                 # Assume an unicode message but mixed-up args
                 msg = self.msg.encode('utf-8', 'replace')
-                return msg.format(*self.args, **self.kwargs)
+                return self._format_message(msg, *self.args, **self.kwargs)
             except (UnicodeEncodeError, AttributeError):
                 # we catch AttributeError since if msg is bytes, it won't have the 'format' method
                 if sys.exc_info()[0] is AttributeError and (PY2 or not isinstance(self.msg, bytes)):
@@ -517,7 +523,7 @@ class LogRecord(object):
                 # but this codepath is unlikely (if the message is a constant
                 # string in the caller's source file)
                 msg = self.msg.decode('utf-8', 'replace')
-                return msg.format(*self.args, **self.kwargs)
+                return self._format_message(msg, *self.args, **self.kwargs)
 
         except Exception:
             # this obviously will not give a proper error message if the
@@ -638,7 +644,7 @@ class LogRecord(object):
         """The formatted exception which caused this record to be created
         in case there was any.
         """
-        if self.exc_info is not None:
+        if self.exc_info is not None and self.exc_info != (None, None, None):
             rv = ''.join(traceback.format_exception(*self.exc_info))
             if PY2:
                 rv = rv.decode('utf-8', 'replace')
@@ -877,25 +883,9 @@ class RecordDispatcher(object):
             if not handler.should_handle(record):
                 continue
 
-            # a filter can still veto the handling of the record.  This
-            # however is already operating on an initialized and processed
-            # record.  The impact is that filters are slower than the
-            # handler's should_handle function in case there is no default
-            # handler that would handle the record (delayed init).
-            if handler.filter is not None \
-               and not handler.filter(record, handler):
-                continue
-
-            # if this is a blackhole handler, don't even try to
-            # do further processing, stop right away.  Technically
-            # speaking this is not 100% correct because if the handler
-            # is bubbling we shouldn't apply this logic, but then we
-            # won't enter this branch anyways.  The result is that a
-            # bubbling blackhole handler will never have this shortcut
-            # applied and do the heavy init at one point.  This is fine
-            # however because a bubbling blackhole handler is not very
-            # useful in general.
-            if handler.blackhole:
+            # first case of blackhole (without filter).
+            # this should discard all further processing and we don't have to heavy_init to know that...
+            if handler.filter is None and handler.blackhole:
                 break
 
             # we are about to handle the record.  If it was not yet
@@ -907,6 +897,22 @@ class RecordDispatcher(object):
                 self.process_record(record)
                 record_initialized = True
 
+
+            # a filter can still veto the handling of the record.  This
+            # however is already operating on an initialized and processed
+            # record.  The impact is that filters are slower than the
+            # handler's should_handle function in case there is no default
+            # handler that would handle the record (delayed init).
+            if handler.filter is not None \
+               and not handler.filter(record, handler):
+                continue
+
+            # We might have a filter, so now that we know we *should* handle
+            # this record, we should consider the case of us being a black hole...
+            if handler.blackhole:
+                break
+
+
             # handle the record.  If the record was handled and
             # the record is not bubbling we can abort now.
             if handler.handle(record) and not handler.bubble:
